//core files
exec("./maps.cs");
exec("./loadout.cs");
exec("./rounds.cs");
exec("./commands.cs");
exec("./help.cs");
exec("./package.cs");
exec("./debug.cs");
exec("./events.cs");

function DM_Tick(%ticks, %first) //main tick, controls and is manipulated by rounds and the minigame
{
	cancel($DM::Tick);
	$DM::RoundRestarting = false;
	
	if($DefaultMiniGame.numMembers > 0)
	{
		//make sure rounds play out if they had players in them
		%ticks--;
		%secs = %ticks * 0.1;
		$DM::RoundActive = true;
	}
	else
	{
		if($DM::RoundActive)
		{
			//make sure its an int and not 0 or below
			$Pref::Server::DeathMatch::RoundLimit = mFloor($Pref::Server::DeathMatch::RoundLimit);
			if($Pref::Server::DeathMatch::RoundLimit <= 0)
				$Pref::Server::DeathMatch::RoundLimit = 1;
			
			$DM::ResetCount = $Pref::Server::DeathMatch::RoundLimit; //trigger a map rotation at round end
			%ticks = 0; //force the round to end
		}
	}
	
	//if it's not active, the round will last forever (until someone joins)
	//this is so the server doesnt constantly change maps or start new rounds at points where there are no players
	if($DM::RoundActive)
	{
		if(%secs == mfloor(%secs))
		{
			if(%secs <= 0)
			{
				echo("Round ending...");
				$DM::RoundRestarting = true;
				
				for(%a = 0; %a < $DefaultMinigame.numMembers; %a++)
				{
					%member = $DefaultMinigame.member[%a];
					if(isObject(%member))
					{
						//clear hud
						%member.setDeathMatchBottomPrintInfo();

						//reset key stuff
						%member.DMHasKey_Red = false;
						%member.DMHasKey_Yellow = false;
						%member.DMHasKey_Green = false;
						%member.DMHasKey_Blue = false;
						
						//free camera
						%camera = %member.camera;
						%camera.setFlyMode();
						%camera.mode = "Observer";
						%member.setControlObject(%camera);
					}
				}
				
				DM_doRoundEnd();
				$DM::RoundActive = false;
				$DefaultMiniGame.scheduleReset();
				return;
			}
			
			if(%secs <= 5 ||
			   %secs == 10 ||
			   %secs == 30 ||
			   %secs == 60 ||
			   %secs == 90 ||
			   %secs == 120 ||
			   %secs == 300 ||
			   %secs == 600 ||
			   %secs == 1200 ||
			   %secs == 1800 ||
			   %secs == 3600 ||
			   %first == true)
			{
				messageAll('', "\c6" @ getTimeString(%secs + %first) @ " remaining");
				%first = false;
			}
			
			$DM:CurrentTime = %secs;
		}

		for(%a = 0; %a < $DefaultMinigame.numMembers; %a++)
			%member = $DefaultMinigame.member[%a];
	}
	
	$DM::Tick = schedule(100, 0, DM_Tick, %ticks, %first);
}

function DM_TickAlt() //tick to do other things, runs independently of rounds and the minigame
{
	cancel($DM::TickAlt);
	
	for(%a = 0; %a < $DefaultMinigame.numMembers; %a++)
	{
		%member = $DefaultMinigame.member[%a];
		if(isObject(%member))
			%member.setDeathMatchBottomPrintInfo(); //apply hud to players
	}
	
	$DM::TickAlt = schedule(100, 0, DM_TickAlt);
}

function gameConnection::setDeathMatchBottomPrintInfo(%this)
{
	if($DM::ResetCount <= 0) //if a map is loading
	{
		//loading message
		%bottomPrint = "<just:center>";
		%bottomPrint = %bottomPrint @ "\c6Loading \c3" @ DM_getMapName($DM::CurrentMap) @ "\c6...";
	}
	else if(!$DM::RoundRestarting) //if the round is not resetting / if its in normal gameplay
	{
		//left
		%bottomPrint = "<just:left>";
		
		//health
		if(isObject(%this.player))
		{
			%player = %this.player;
			%data = %player.getDatablock();
			%this.maxHealth = %data.maxDamage;
			%this.currHealth = %data.maxDamage - %player.getDamageLevel();
		}
		else
			%this.currHealth = 0;
		
		//get max health
		if(%this.maxHealth > 0)
		{
			%bottomPrint = %bottomPrint @ "\c6Health";
			
			if(mceil(%this.currHealth) > %this.maxHealth)
				%bottomPrint = %bottomPrint @ "\c4: " @ mceil(%this.currHealth) @ "\c6/\c4" @ %this.maxHealth;
			else if(mceil(%this.currHealth) > 90)
				%bottomPrint = %bottomPrint @ "<color:33ff00>: " @ mceil(%this.currHealth) @ "\c6/<color:33ff00>" @ %this.maxHealth;
			else if(mceil(%this.currHealth) > 80)
				%bottomPrint = %bottomPrint @ "<color:66ff00>: " @ mceil(%this.currHealth) @ "\c6/<color:66ff00>" @ %this.maxHealth;
			else if(mceil(%this.currHealth) > 70)
				%bottomPrint = %bottomPrint @ "<color:99ff00>: " @ mceil(%this.currHealth) @ "\c6/<color:99ff00>" @ %this.maxHealth;
			else if(mceil(%this.currHealth) > 60)
				%bottomPrint = %bottomPrint @ "<color:ccff00>: " @ mceil(%this.currHealth) @ "\c6/<color:ccff00>" @ %this.maxHealth;
			else if(mceil(%this.currHealth) > 50)
				%bottomPrint = %bottomPrint @ "\c3: " @ mceil(%this.currHealth) @ "\c6/\c3" @ %this.maxHealth;
			else if(mceil(%this.currHealth) > 40)
				%bottomPrint = %bottomPrint @ "<color:ffcc00>: " @ mceil(%this.currHealth) @ "\c6/<color:ffcc00>" @ %this.maxHealth;
			else if(mceil(%this.currHealth) > 30)
				%bottomPrint = %bottomPrint @ "<color:ff9900>: " @ mceil(%this.currHealth) @ "\c6/<color:ff9900>" @ %this.maxHealth;
			else if(mceil(%this.currHealth) > 20)
				%bottomPrint = %bottomPrint @ "<color:ff6600>: " @ mceil(%this.currHealth) @ "\c6/<color:ff6600>" @ %this.maxHealth;
			else if(mceil(%this.currHealth) > 10)
				%bottomPrint = %bottomPrint @ "<color:ff3300>: " @ mceil(%this.currHealth) @ "\c6/<color:ff3300>" @ %this.maxHealth;
			else
				%bottomPrint = %bottomPrint @ "\c0: " @ mceil(%this.currHealth) @ "\c6/\c0" @ %this.maxHealth;
		}
		
		//right
		%bottomPrint = %bottomPrint @ "<just:right>";
		
		//keys
		if(%this.DMHasKey_Red)
			%bottomPrint = %bottomPrint @ "\c0KEY ";
		if(%this.DMHasKey_Yellow)
			%bottomPrint = %bottomPrint @ "\c3KEY ";
		if(%this.DMHasKey_Green)
			%bottomPrint = %bottomPrint @ "\c2KEY ";
		if(%this.DMHasKey_Blue)
			%bottomPrint = %bottomPrint @ "\c1KEY ";
	}
	else
		%bottomPrint = ""; //set it to blank, will clear the bottom print during round transitions
	
	//completed building it, send it.
	commandToClient(%this, 'bottomPrint', %bottomPrint, 0, true);
}