function DM_BuildMapList()
{
	%pattern = "Add-Ons/DeathMatch_*/save.bls";
	
	$DM::numMaps = 0;
	
	%file = findFirstFile(%pattern);
	while(%file !$= "")
	{
		$DM::Map[$DM::numMaps] = %file;
		$DM::numMaps++;
		%file = findNextFile(%pattern);
	}
}

function DM_getMapName(%displayName, %fromPath)
{
	%error = %displayName;
	
	if(!%fromPath)
		%displayName = $DM::Map[%displayName];
	
	%displayName = strReplace(%displayName, "Add-Ons/DeathMatch_", "");
	%displayName = strReplace(%displayName, "/save.bls", "");
	%displayName = strReplace(%displayName, "_", " ");
	
	if(%displayName $= "")
		error(%fromPath @ "Tried to get display name with " @ %error);
	
	return %displayName;
}

function DM_NextMap()
{
	echo("Loading next map...");
	
	if(($Pref::Server::DeathMatch::RandomMaps == true && !$DM::BypassRandom) || $DM::ForceRandom)
	{
		%i = getRandom(1, $DM::numMaps);

		if(%i <= $DM::CurrentMap)
			%i--;
		
		$DM::CurrentMap = %i - 1;
		$DM::ForceRandom = false;
	}
	
	$DM::BypassRandom = false;
	
	$DM::CurrentMap = mFloor($DM::CurrentMap);
	$DM::CurrentMap++;
	$DM::CurrentMap = $DM::CurrentMap % $DM::numMaps;
	
	cancel($DM::Tick);
	$DM::RoundRestarting = true;
	$DM::ResetCount = 0;
	DM_LoadMap_Phase1($DM::Map[$DM::CurrentMap]);
}

function DM_LoadMap_Phase1(%filename)
{
	//suspend minigame resets
	$DM::MapChange = 1;

	//put everyone in observer mode
	%mg = $DefaultMiniGame;
	if(!isObject(%mg))
	{
		error("ERROR: DM_LoadMap_Phase1(" @ %filename  @ ") - default minigame does not exist");
		return;
	}
	for(%i = 0; %i < %mg.numMembers; %i++)
	{
		%client = %mg.member[%i];
		%player = %client.player;
		if(isObject(%player))
			%player.delete();

		%camera = %client.camera;
		%camera.setFlyMode();
		%camera.mode = "Observer";
		%client.setControlObject(%camera);
	}
	
	//clear all bricks 
	// note: this function is deferred, so we'll have to set a callback to be triggered when it's done
	BrickGroup_888888.chaindeletecallback = "DM_LoadMap_Phase2(\"" @ %filename @ "\");";
	BrickGroup_888888.chaindeleteall();
}

function DM_LoadMap_Phase2(%filename)
{
	echo("Loading deathmatch map from " @ %filename);
	%loadMsg = "\c5Now loading \c6" @ DM_getMapName(%filename, 1);
	
	//load config file
	$DM::Item0 = 0;
	$DM::Item1 = 0;
	$DM::Item2 = 0;
	$DM::Item3 = 0;
	$DM::Item4 = 0;
	$DM::Item0alt = 0;
	$DM::Item1alt = 0;
	$DM::Item2alt = 0;
	$DM::Item3alt = 0;
	$DM::Item4alt = 0;
	$DM::BrickDamage = false;
	$DM::killWater = false;
	$DM::allowAltLoadout = false;
	$DM::mapCredits = "NONE";
	$DM::mapDescription = "NONE";
	$DM::mapEnvironment = false;
	$DM::mapFile = %fileName;
	
	%configFilename = filePath(%fileName) @ "/config.txt";
	if(isFile(%configFilename))
	{
		%file = new fileObject();
		%file.openForRead(%configFilename);
		echo(" +- has config file");
		
		while(!%file.isEOF())
		{
			%line = trim(%file.readLine());

			if(%line $= "")
				continue;

			%firstWord = getWord(%line, 0);

			switch$(%firstWord)
			{
				case "item0":
					%item0 = getWord(%line, getWordCount(%line) - 1);
				case "item1":
					%item1 = getWord(%line, getWordCount(%line) - 1);
				case "item2":
					%item2 = getWord(%line, getWordCount(%line) - 1);
				case "item3":
					%item3 = getWord(%line, getWordCount(%line) - 1);
				case "item4":
					%item4 = getWord(%line, getWordCount(%line) - 1);
				case "item0alt":
					%item0alt = getWord(%line, getWordCount(%line) - 1);
				case "item1alt":
					%item1alt = getWord(%line, getWordCount(%line) - 1);
				case "item2alt":
					%item2alt = getWord(%line, getWordCount(%line) - 1);
				case "item3alt":
					%item3alt = getWord(%line, getWordCount(%line) - 1);
				case "item4alt":
					%item4alt = getWord(%line, getWordCount(%line) - 1);
				case "brickDamage":
					%brickDamage = getWord(%line, getWordCount(%line) - 1);
				case "killWater":
					%killWater = getWord(%line, getWordCount(%line) - 1);
				case "allowAltLoadout":
					%allowAltLoadout = getWord(%line, getWordCount(%line) - 1);
			}
		}

		if(%item0 !$= "")
			$DM::Item0 = %item0;
		if(%item1 !$= "")
			$DM::Item1 = %item1;
		if(%item2 !$= "")
			$DM::Item2 = %item2;
		if(%item3 !$= "")
			$DM::Item3 = %item3;
		if(%item4 !$= "")
			$DM::Item4 = %item4;
		if(%item0alt !$= "")
			$DM::Item0alt = %item0alt;
		if(%item1alt !$= "")
			$DM::Item1alt = %item1alt;
		if(%item2alt !$= "")
			$DM::Item2alt = %item2alt;
		if(%item3alt !$= "")
			$DM::Item3alt = %item3alt;
		if(%item4alt !$= "")
			$DM::Item4alt = %item4alt;
		if(%allowAltLoadout $= "true" || mfloor(%allowAltLoadout) >= 1)
			$DM::allowAltLoadout = true;
		
		if(%brickDamage $= "true" || mfloor(%brickDamage) >= 1)
			$DM::BrickDamage = true;
		
		if(%killWater $= "true" || mfloor(%killWater) >= 1)
			$DM::killWater = true;
		
		%file.close();
		%file.delete();
	}
	else
		echo(" +- no config file");
	
	//read and display credits file, if it exists
	// limited to one line
	%creditsFilename = filePath(%fileName) @ "/credits.txt";
	if(isFile(%creditsFilename))
	{
		%file = new FileObject();
		%file.openforRead(%creditsFilename);

		%line = %file.readLine();
		%line = stripMLControlChars(%line);
		if(%line !$= "")
		{
			echo(" +- has credits file");
			%loadMsg = %loadMsg @ "\c5, created by \c3" @ %line;
			$DM::mapCredits = "\c5Created by \c3" @ %line;
		}
		else
			echo(" +- has credits file, but it's blank");

		%file.close();
		%file.delete();
	}
	else
		echo(" +- no credits file");
	messageAll('', %loadMsg @ "\c5.");
	
	//read and display description file, if it exists
	// limited to one line
	//blockland glass forces descriptions to be in addons, sigh
	%descriptionFilename = filePath(%fileName) @ "/description.txt";
	if(isFile(%descriptionFilename))
	{
		%file = new FileObject();
		%file.openforRead(%descriptionFilename);

		%line = %file.readLine();
		%line = stripMLControlChars(%line);
		if(%line !$= "")
		{
			echo(" +- has description file");
			%descMsg = %descMsg @ "\c5" @ %line;
			$DM::mapDescription = "\c5" @ %line;
		}
		else
			echo(" +- has description file, but it's blank");

		%file.close();
		%file.delete();
	}
	else
		echo(" +- no description file");
	messageAll('', %descMsg);
	
	//load environment if it exists
	%envFile = filePath(%fileName) @ "/environment.txt"; 
	if(isFile(%envFile))
	{
		//echo("parsing env file " @ %envFile);
		//usage: GameModeGuiServer::ParseGameModeFile(%filename, %append);
		//if %append == 0, all minigame variables will be cleared 
		%res = GameModeGuiServer::ParseGameModeFile(%envFile, 1);
		echo(" +- has environment file");

		EnvGuiServer::getIdxFromFilenames();
		EnvGuiServer::SetSimpleMode();
		$DM::mapEnvironment = true;
		
		if(!$EnvGuiServer::SimpleMode)
		{
			EnvGuiServer::fillAdvancedVarsFromSimple();
			EnvGuiServer::SetAdvancedMode();
		}
	}
	else
		echo(" +- no environment file");
	
	//create fake client so we can change some minigame and environment stuff
	if(!isObject(DM_FakeClient))
		new GameConnection(DM_FakeClient);
	
	//make it admin
	DM_FakeClient.isAdmin = true;
	DM_FakeClient.isSuperAdmin = true;
	DM_FakeClient.minigame = $DefaultMinigame;
	
	//set water height to a set number so kill water works
	if($DM::killWater)
		serverCmdEnvGui_SetVar(DM_FakeClient, "WaterHeight", 5);
	
	//set brick damage
	serverCmdSetMiniGameData(DM_FakeClient, "BD" SPC $DM::BrickDamage);
	
	//set fake name tables so modded items/vehicles are replaced with defaults
	DM_AddFakeNameTables();
	
	//load save file
	schedule(10, 0, serverDirectSaveFileLoad, %fileName, 3, "", 2, 1);
}